        title ""
;;; *********************************************************************
;;; Filename:	main.asm
;;; Purpose:	Main loop for Rotary Display Control
;;; Revision:	0.5
;;; Date:	March 21, 2010
;;; Author:	Patrick Boyd
;;; Copyright:	
;;; *********************************************************************
;;; Revision History:
;;;	
;;; *********************************************************************


;;; *********************************************************************
;;; Assembler Directives, Includes and Defines
;;; *********************************************************************
;{
	list 			; Turn on list output.

	;; Include register, bit, and other info specific to 
	;; the specified device.	
	ifdef	__16F873
 	#include <p16f873.inc>
	messg "Assembling for PIC16F873."
	endif
	ifdef	__16F874
 	#include <p16f874.inc>
	messg "Assembling for PIC16F874."
	endif
	ifdef	__16F876
 	#include <p16f876.inc>
	messg "Assembling for PIC16F876."
	endif
	ifdef	__16F877
 	#include <p16f877.inc>
	messg "Assembling for PIC16F877."
	endif
	ifdef	__16F873A
	#define A_Device	
 	#include <p16f873a.inc>
	messg "Assembling for PIC16F873A."
	endif
	ifdef	__16F874A
 	#include <p16f874a.inc>
	#define A_Device	
	messg "Assembling for PIC16F874A."
	endif
	ifdef	__16F876A
	#define A_Device	
 	#include <p16f876a.inc>
	messg "Assembling for PIC16F876A."
	endif
	ifdef	__16F877A
	#define A_Device	
 	#include <p16f877a.inc>
	messg "Assembling for PIC16F877A."
	endif

	#define SI1 0
	#define SI2 1
	#define LE	2
	#define OE	3
	#define CLOCK	1
	
	#define SCROLL_DELAY 0x78
	#define MAX_CHARS 0x1C
	#define MAX_COLS 0xE0

	errorlevel -302
	
	;}


	

	




	
;;; ---------------------------------------------------------------------
;;; Variable Address Assignments.
;;; ---------------------------------------------------------------------
	;{
	UDATA			; Start of uninitialized data section.
				; Reserve memory for variables using the
				; "RES" directive here.  Note that the
				; linker will assign addresses.
				; eg)  ADcounter  res  1
				;      (reserves 1 byte for ADcounter).



	UDATA_OVR		; Uninitialized data overlay section.
				; Upon occasion, temporary variables
				; can be over-written by other temporary
				; variables.  These would appear here.
				; BUT!  If you ever intend on time-sliced
				; or pre-emptive multitasking		 
				; don't *ever* use this directive!

	UDATA_SHR		; Uninitialized data shared section.
				; Variables that appear here appear
				; across all banks, meaning that no
				; bank selection will be required.
				; Variables of this type are *required*
				; when saving CPU context when an
				; interrupt occurs.  Not all devices
				; have shared memory.  In that case, you
				; need to reserve memory across individual
				; banks at the same offset by using
				; multiple UDATA or UDATA_SHR assignments
				; with an optional address specified.
W_Context:	res 1		; Context saving on interrupt:	W.

	;; For saving context.
STATUS_Context:	res 1		; Saving of STATUS on interrupt.
PCLATH_Context:	res 1		; Saving of PCLATH on interrupt.

	;;	Output to be displayed
OUTPUT_H:	res 1
OUTPUT_L:	res 1

	;; 	Registers for storing temp values for output
TEMP_H:		res 1
TEMP_L:		res 1
COL_DONE:	res 1

	;; Register for delay loop
COUNT:		res 1

	;; Registers for data writes
DATA1:		res 1
DATA2:		res 1
DATA3:		res 1
DATA4: 		res 1

ADDR:		res 1

;; Registers for storing message position information

MSG_COUNT:		res 1
OFFSET:			res 1
LEAD_SPACES		res 1
ROTATIONS:		res 1
CUR_COL:		res 1

;}



;;; ---------------------------------------------------------------------
;;; Establish the OPTION register bit values. 
;;; ---------------------------------------------------------------------
;{

;;; The '__CONFIG' directive is used to embed PIC configuration data 
;;; within an assembly file. The labels following the directive 
;;; are located in the .inc file.  See the device data sheet for
;;; additional information on the configuration word.
	ifndef A_Device	
		__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _WRT_ENABLE_ON & _LVP_OFF & _CPD_OFF
	endif
;;; _CP_OFF:	turn off code protection.  Don't change this unless you
;;;             want a device that can never be programmed again.  This
;;;             is a "bug" in some PIC devices.
;;; _WDT_OFF:	turn off the watchdog timer.
;;; _BODEN_ON:	turn on power brown-out reset.
;;; _PWRTE_ON:	turn on power-up timer.
;;; _XT_OSC:	specify that the device is using an XT oscillator.
;;; _WRT_ENABLE_ON:  enable writing to data EEPROM.
;;; _LVP_OFF:	disable low-voltage in-circuit programming.
;;; _CPD_OFF:	disable data EEPROM write protection.

	ifdef A_Device
		messg "A revision device."
	 	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _WRT_OFF & _LVP_OFF & _CPD_OFF
	endif

;;; _CP_OFF:	turn off code protection.  Don't change this unless you
;;;             want a device that can never be programmed again.  This
;;;             is a "bug" in some PIC devices.
;;; _WDT_OFF:	turn off the watchdog timer.
;;; _BODEN_ON:	turn on power brown-out reset.
;;; _PWRTE_ON:	turn on power-up timer.
;;; _XT_OSC:	specify that the device is using an XT oscillator.
;;; _WRT_OFF:	disable wrie-prottection of program FLASH.
;;; _LVP_OFF:	disable low-voltage in-circuit programming.
;;; _CPD_OFF:	disable data EEPROM write protection.
	;}
;;; --------------------------------------------------------------------
;;; Main Program.
;;; --------------------------------------------------------------------
;;; --------------------------------------------------------------------
;;; Interrupt Handlers
;;; --------------------------------------------------------------------

ISR:	CODE	0x0004		; Location of interrupt vector.
;{

	;; Save the CPU context.
	;; Note that W_Context was declared under the UDATA_SHR section.
	;; In the PIC16F876/7, it will appear across all banks of memory 
	;; and therefore no bank switching is required. For the
	;; PIC16F873/4, a linker script and use of the UDATA_SHR section
	;; reserves space for W_Context across all banks, even though
	;; there are actually two separate locations.  This code takes
	;; this possible separation into account.  See the linker script
	;; (16F873.lkr), available from the same location as this
	;; template for how this allocation occurs.
	
	movwf	W_Context
	swapf	STATUS,W
	banksel	STATUS_Context	
	movwf	STATUS_Context
	movf	PCLATH,W
	movwf	PCLATH_Context

	;; Why the swapf instruction, above?  That's a good question.
	;; The reason is that, unlike a move instruction, swapf doesn't
	;; affect any bits in STATUS, which is important!
	;; There are situations where you will need to save (and also
	;; restore more than just these two registers.

	;; ------------------------------------------------------------
	;; Your interrupt service routine code goes here.
	;; ------------------------------------------------------------
	;; You'll generally need to follow these steps:
	;; 1.  Determine what caused the interrupt by checking
	;;     appropriate interrupt flags.  You'll usually know which
	;;     ones to check, since you enable interrupts from certain
	;;     peripherals in the main program.
	;; 2.  Take appropriate action.
	;; 3.  Clear the interrupt source.  How to do this depends
	;;     on the peripheral that caused the interrupt, but sometimes
	;;     it is as simple as clearing a flag.
	;; ------------------------------------------------------------

	;btfsc	INTCON, TMR0IF	; Check to see if it was the timer that caused the interrupt
	;goto	TMR0_ISR

EXT_ISR:
	btfsc	COL_DONE, 0
	goto	TurnOff
TurnOn:
	movlw	0xFF
	movwf	OUTPUT_H
	movwf	OUTPUT_L
	call	OUTPUT
	bsf		COL_DONE, 0
	bcf		INTCON, INTF	; Clear the interrupt flag
	goto	ISR_Restore
	
TurnOff:
	movlw	0x00
	movwf	OUTPUT_H
	movwf	OUTPUT_L
	call	OUTPUT
	bcf		COL_DONE, 0
	bcf		INTCON, INTF
	





	;; Come back to this point when your interrupt processing is
	;; complete.
ISR_Restore:
	;; Restore the CPU context.
	banksel PCLATH_Context
	movf	PCLATH_Context,W
	movwf	PCLATH
	swapf	STATUS_Context,W
	movwf	STATUS		; Restores whichever bank W_Context
				; actually appears in.
	swapf	W_Context,F	; See the note above as to why swapf
	swapf	W_Context,W	; is used.

	;; Return from the interrupt, which pulls an address off the
	;; (invisible) stack.
	retfie
	;}

	;; ------------------------------------------------------------	
	;; Your main code starts here.  If it just follows the ISR, that
	;; is usually OK.
	;; ------------------------------------------------------------	
	CODE

Main:
	;; Program initialization.
;{
	;;;;;
	;;;-----------------------------------------
	; Switch To Bank 4
	;;;-----------------------------------------	
	;;;;;
	banksel EECON1 	; Move to the bank with TRISB
	
	;;;-----------------------------------------
	; Set up EECON1
	;;;-----------------------------------------
	; EEPGD - 1, Use program memory
	; 6:4 - 0, Unimplemented
	; WRERR - 0 Clear Error Bit
	; WREN - 0, Writes not enabled
	; WR - 0, Not writing yet
	; RD - 0, Not reading yet
	;;;-----------------------------------------

	movlw	b'10000000'
	movwf	EECON1
	;;;;;
	;;;-----------------------------------------
	; Switch To Bank 1
	;;;-----------------------------------------	
	;;;;;
	banksel TRISB 	; Move to the bank with TRISB

	;;;-----------------------------------------
	; Set up the TRISA register as follows
	;;;-----------------------------------------
	; RA0 - Output
	; RA1 - Output
	; RA2 - Output
	; RA3 - Output
	; RA4 - Output
	; RA5 - Output
	;;;-----------------------------------------

	clrf TRISA

	;;;-----------------------------------------
	; Set up the TRISB register as follows
	;;;-----------------------------------------
	; RB0 - Input
	; RB1 - Output
	; RB2 - Output
	; RB3 - Output
	; RB4 - Output
	; RB5 - Output
	; RB6 - Output
	; RB7 - Output
	;;;-----------------------------------------

	movlw 	0x01
	movwf 	TRISB

	;;;-----------------------------------------
	; Set up the Options register as follows
	;;;-----------------------------------------
	; RBPU - 1, Pull-ups disabled
	; INTEDG - 0, Trigger on falling edge
	; T0CS	- 0, Use internal clock
	; T0SE	- 0, low-to-high ignored 
	; PSA - 0, PSA assigned to Timer0
	; PS2:PS0 - 111, 1:256 Prescale ratio
	;;;-----------------------------------------
	movlw	b'10000111'
	movwf	OPTION_REG

	;;;;;
	;;;-----------------------------------------
	; Switch To Bank 0
	;;;-----------------------------------------	
	;;;;;

	banksel T1CON

	;;;-----------------------------------------
	; Set up the INTCON register as follows
	;;;-----------------------------------------
	; GIE - 0, Leave Interrupts off for now
	;		   will turn on when initialization done
	; PEIE - 1, Enable Peripheral Interrupts
	; TMR0IE - 1, Enable Timer0 overflow interrupt
	; INTE - 1, Enable RB0/INT External Interrupt
	; RBIE - 0 , Disable RB Port Change Interrupt
	; TMR01F - 0, Clear the Timer0 overflow interrupt flag
	; INTF - 0, Clear the RB0/INT External Interrupt flag
	; RBIF - 0, Clear the RB port Change flag
	;;;-----------------------------------------
	movlw b'01110000'
	movwf INTCON

	;;;-----------------------------------------
	; Set up Timer1
	;;;-----------------------------------------
	; T1CKPS1:0 - 00, 1:1 Prescale Ratio
	; T1OSCEN - 0, Turn Oscillator Off
	; T1SYNC - 0, External Clock sync, ignored
	; TMR1CS - 0, Use Internal Clock 
	; TMR1ON - 0, Turn on at beginning of main loop
	;;;-----------------------------------------
	clrf	T1CON

	;;;-----------------------------------------
	; Clear Timer1 Registers
	;;;-----------------------------------------	
	clrf	TMR1H
	clrf	TMR1L


	;;;-----------------------------------------
	; Set up PORT variables and clear pins
	;;;-----------------------------------------

	banksel	PORTA 	; Select the bank with PORTA, since we'll be using it

	clrf	PORTA
	clrf	PORTB

	;;;-----------------------------------------
	; Other things not being used any more
	;;;-----------------------------------------

	;clrf 	PIE1	; clear the peripheral interrupt fields
	;bsf	PIE1, TMR1IE	; enable the timer1 overflow interrupt

	;clrf	PIR1

	;;;-----------------------------------------
	; Initialize Memory
	;;;-----------------------------------------

	movlw	0x00
	movwf	ADDR
	clrf	COL_DONE
	;call INIT		; sets up all the messages in program memory

	;;;-----------------------------------------
	; Final Steps
	;;;-----------------------------------------

	bsf		T1CON, TMR1ON	; Start the Timer
	bsf		INTCON, GIE		; Enable all interrupts
;}

	goto	DebugLoop	;; Jump to whatever main code you want to run
	
;;;	-------------------------------------------------------------------------------------
;;;	For Reference Here are the list of which pins on the PIC
;;;	are associated with which LED driver lines
;;;	RB0 - Hall Sensor
;;;	RA0 - Serial In 1
;;;	RA1 - Serial In 2
;;; RA2 - Latch Enable
;;;	RA3 - Output Enable
;;;	RB1 - Clock
;;; -------------------------------------------------------------------------------------

;;; ********************************************************************
;;; MainLoop
;;; Description: The main process loop
;;; Requires: Nothing!
;;; Returns: I really hope it never returns...
;;; Locations Affected:	Everything...
;;;	Notes: The main loop is responsible for the following things
;;; 		in the following order of importance:
;;;			1) Load new values for the LEDs to display
;;;			2) Scroll the message by updating offset and leading spaces
;;;			3) ???
;;;			4) Profit
;;; ********************************************************************



Foobar:
	banksel	PORTA
	movlw	0x07
	movwf	PORTA
	
	bsf		PORTB, CLOCK
	bcf		PORTB, CLOCK
	
	bsf		PORTB, CLOCK
	bcf		PORTB, CLOCK
	
	bsf		PORTB, CLOCK
	bcf		PORTB, CLOCK
	
	bsf		PORTB, CLOCK
	bcf		PORTB, CLOCK
	
	bsf		PORTB, CLOCK
	bcf		PORTB, CLOCK
	
	bsf		PORTB, CLOCK
	bcf		PORTB, CLOCK
	
	bsf		PORTB, CLOCK
	bcf		PORTB, CLOCK
	
	bsf		PORTB, CLOCK
	bcf		PORTB, CLOCK
	
	bsf		PORTB, CLOCK
	
	goto	DebugLoop

Debug:
	movlw	0xFF
	movwf	OUTPUT_H
	movwf	OUTPUT_L
	call	OUTPUT

DebugLoop:
	nop
	nop
	goto 	DebugLoop


MainLoop:


	btfsc	COL_DONE, 0			;; check to see if the current column has 
	goto	DataLoad			;; been displayed
	goto	ScrollTest
	
DataLoad:
	
	
	
ScrollTest:
	movf	ROTATIONS, W		;; Check if enough rotations have passed
	sublw	SCROLL_DELAY		;; to scroll the message
	btfsc	STATUS, Z
	goto	Scroll
	goto	MainLoopEnd
	
Scroll:
	clrf	ROTATIONS			;; Reset rotations count
	movf	OFFSET, F			;; Check to see if offset is 0
	btfsc	STATUS, Z			;; meaning the entire message is on the
	goto	AddLeadingSpace		;; display
DecOffset:
	decf	OFFSET, F			
	goto	MainLoopEnd
AddLeadingSpace:
	movf	LEAD_SPACES, W		;; check to see if the leading spaces
	sublw	MAX_CHARS			;; have pushed the message off the screen
	btfsc	STATUS, Z			
	goto	ResetOffset			;; if so reset the offset
	incf	LEAD_SPACES, F
	goto	MainLoopEnd

ResetOffset:
	clrf	LEAD_SPACES
	movf	MSG_COUNT, W
	sublw	0x01
	movwf	OFFSET
	
	

MainLoopEnd:	
	
	goto MainLoop ; Return to the start of the loop


	;; Don't let execution fall down to here, otherwise the
	;; subroutines (or whatever follows) will be executed without
	;; parameters, and chaos reigns when a "return" instruction
	;; executes with no return address on the stack!
	
;;; --------------------------------------------------------------------
;;; Application-specific Subroutines.
;;; --------------------------------------------------------------------
;;; ********************************************************************
;;; Subroutine Name:	
;;; Description:	
;;; Requires:		
;;; Returns:		
;;; Locations Affected:	
;;; ********************************************************************

;;; ********************************************************************
;;; Subroutine Name: DATA_Write
;;; Description: Writes a value to Data flash memory
;;; Requires: EEDATA must have the data to be written
;;; Returns: Nothing
;;; Locations Affected:	Destroys w
;;; ********************************************************************
DATA_Write:
;{	
	banksel 	EECON1
	btfsc		EECON1, WR
	goto		$-1

	bcf			EECON1, EEPGD
	bsf			EECON1, WREN

	movlw		0x55
	movwf		EECON2
	movlw		0xAA
	movwf		EECON2
	bsf			EECON1, WR

	bcf			EECON1, WREN
	
	RETURN
;}

;;; ********************************************************************
;;; Subroutine Name: DATA_Read
;;; Description: Reads a value from Data flash memory
;;; Requires: EEADR to be loaded with the desired memory location
;;; Returns: The read data in EEDATA
;;; Locations Affected:	Destroys w
;;; ********************************************************************
DATA_Read:
;{
	banksel 	EECON1
	
	bcf			EECON1, EEPGD
	bsf			EECON1, RD


	RETURN
	
	;}


;;; ********************************************************************
;;; Subroutine Name: PROGRAM_Write
;;; Description: Writes a value to PROGRAM flash memory
;;; Requires: Initial Address loaded into EADDR and EADDRH
;;;			  Data1:4 loaded with values to be stored
;;; Returns: Nothing
;;; Locations Affected:	Destroys w
;;;	NOTE: NOT WORKING
;;; ********************************************************************

PROGRAM_Write:
	
;{
	banksel 	EECON1

	bsf			EECON1, EEPGD
	bsf			EECON1, WREN

	banksel		EEDATA

	movf		DATA1, W		;;;  1
	movwf		EEDATA
	clrw
	movwf		EEDATH

	banksel		EECON1

	movlw		0x55
	movwf		EECON2
	movlw		0xAA
	movwf		EECON2
	bsf			EECON1, WR
	nop
	nop

	bcf			EECON1, WREN

	banksel		EEDATA

	incf		EEADR, F


	movf		DATA2, W		;;;  2
	movwf		EEDATA
	clrw
	movwf		EEDATH

	banksel		EECON1

	movlw		0x55
	movwf		EECON2
	movlw		0xAA
	movwf		EECON2
	bsf			EECON1, WR
	nop
	nop

	bcf			EECON1, WREN

	banksel		EEDATA

	incf		EEADR, F


	movf		DATA3, W		;;;  3
	movwf		EEDATA
	clrw
	movwf		EEDATH

	banksel		EECON1

	movlw		0x55
	movwf		EECON2
	movlw		0xAA
	movwf		EECON2
	bsf			EECON1, WR
	nop
	nop

	bcf			EECON1, WREN

	banksel		EEDATA

	incf		EEADR, F


	movf		DATA4, W		;;;  4
	movwf		EEDATA
	clrw
	movwf		EEDATH

	banksel		EECON1

	movlw		0x55
	movwf		EECON2
	movlw		0xAA
	movwf		EECON2
	bsf			EECON1, WR
	nop
	nop

	bcf			EECON1, WREN

	banksel		EEDATA

	incf		EEADR, F
	
	RETURN
	;}

;;; ********************************************************************
;;; Subroutine Name:	INIT_WRITE	
;;; Description:	Actually does the writing
;;; Requires:	The DATA1:4 register to be set up as well as EEADR to be set
;;;				Also, EECON1 must be configured to be pointing at program memory
;;;				and writes to be enabled
;;; Returns:	Nothing
;;; Locations Affected: ?	
;;; ********************************************************************

INIT_WRITE:
;{
	btfsc		ADDR, 7
	goto		INIT_WRITE_EXIT

	banksel		EEDATA

	movf		DATA1, W
	movwf		EEDATA
	movf		ADDR, W
	movwf		EEADR

	call		DATA_Write

	banksel		EEDATA

	incf		ADDR, F

	movf		DATA2, W
	movwf		EEDATA
	movf		ADDR, W
	movwf		EEADR

	call		DATA_Write

	banksel		EEDATA

	incf		ADDR, F

	movf		DATA3, W
	movwf		EEDATA
	movf		ADDR, W
	movwf		EEADR

	call		DATA_Write

	banksel		EEDATA

	incf		ADDR, F

	movf		DATA4, W
	movwf		EEDATA
	movf		ADDR, W
	movwf		EEADR

	call		DATA_Write

	banksel		EEDATA

	incf		ADDR, F

INIT_WRITE_EXIT:

	RETURN



;	banksel		EEDATA	
;
;	movf		DATA1, W
;	movwf		EEDATA			; Load the first byte
;	movlw		0x00
;	movwf		EEDATH
;
;	banksel		EECON1
;	
;	bsf			EECON1, EEPGD
;	bsf			EECON1, WREN
;
;	movlw		0x55				;; The next 5 instructions are necessary
;	movwf		EECON2
;	movlw		0xAA
;	movwf		EECON2
;	bsf			EECON1, WR
;	NOP							;; These two instructions get ignored
;	NOP
;
;	bcf			EECON1, WREN
;
;	banksel		EEDATA
;
;	incf		EEADR, F		; Increment address
;
;	;; Repeat for the other data registers
;
;
;
;	movf		DATA2, W
;	movwf		EEDATA			; Load the first byte
;	movlw		0x00
;	movwf		EEDATH
;
;
;	banksel		EECON1
;
;	bsf			EECON1, EEPGD
;	bsf			EECON1, WREN
;	
;	movlw		0x55				;; The next 5 instructions are necessary
;	movwf		EECON2
;	movlw		0xAA
;	movwf		EECON2
;	bsf			EECON1, WR
;	NOP							;; These two instructions get ignored
;	NOP
;
;	bcf			EECON1, WREN
;
;	banksel		EEDATA
;
;	incf		EEADR, F		; Increment address
;
;	;; Repeat for the other data registers
;
;	movf		DATA3, W
;	movwf		EEDATA			; Load the first byte
;	movlw		0x00
;	movwf		EEDATH
;
;
;	banksel		EECON1
;
;	bsf			EECON1, EEPGD
;	bsf			EECON1, WREN
;	
;	movlw		0x55			;; The next 5 instructions are necessary
;	movwf		EECON2
;	movlw		0xAA
;	movwf		EECON2
;	bsf			EECON1, WR
;	NOP							;; These two instructions get ignored
;	NOP
;
;	bcf			EECON1, WREN
;
;	banksel		EEDATA
;
;	incf		EEADR, F		; Increment address
;
;	;; Repeat for the other data registers
;
;	movf		DATA4, W
;	movwf		EEDATA			; Load the first byte
;	movlw		0x00
;	movwf		EEDATH
;
;
;	banksel		EECON1
;
;	bsf			EECON1, EEPGD
;	bsf			EECON1, WREN
;	
;	movlw		0x55			;; The next 5 instructions are necessary
;	movwf		EECON2
;	movlw		0xAA
;	movwf		EECON2
;	bsf			EECON1, WR
;	NOP							;; These two instructions get ignored
;	NOP
;
;	bcf			EECON1, WREN
;
;	banksel 	EEADR
;
;	incf		EEADR, F		; Increment address
;
;	RETURN
;
;}
;;; ********************************************************************
;;; Subroutine Name: DELAY
;;; Description: Runs through a busy loop to delay a bit	
;;; Requires: Nothing
;;; Returns: Nothing
;;; Locations Affected:	W?
;;; ********************************************************************

DELAY:
;{
	movlw	0xff
	movwf	COUNT

DELAY_LOOP1:
	decfsz	COUNT, 1
	goto	DELAY_LOOP1

	movlw	0xff
	movwf	COUNT
DELAY_LOOP2:
	decfsz	COUNT, 1
	goto	DELAY_LOOP2

	movlw	0xff
	movwf	COUNT
DELAY_LOOP3:
	decfsz	COUNT, 1
	goto	DELAY_LOOP3

	movlw	0xff
	movwf	COUNT
DELAY_LOOP4:
	decfsz	COUNT, 1
	goto	DELAY_LOOP4

	movlw	0xff
	movwf	COUNT
DELAY_LOOP5:
	decfsz	COUNT, 1
	goto	DELAY_LOOP5

	movlw	0xff
	movwf	COUNT
DELAY_LOOP6:
	decfsz	COUNT, 1
	goto	DELAY_LOOP6

	movlw	0xff
	movwf	COUNT
DELAY_LOOP7:
	decfsz	COUNT, 1
	goto	DELAY_LOOP7

	RETURN
;}
;;; ********************************************************************
;;; Subroutine Name: OUTPUT	
;;; Description: Outputs the values stored in OUTPUT_H and OUTPUT_L to the LEDs
;;; Requires: -The values of OUPUT_H and OUTPUT_L to be set
;;;			  -PORTB bank to be selected
;;; Returns: Nothing
;;; Locations Affected:	W, TEMP_H, TEMP_L
;;; ********************************************************************

;;; Right now this is taking 10 Cycles per 2 LEDs, which is 80 cycles for the while thing.
;;; Check later to see if that is too much...

;;;	I took out the porta_base stuff, if it stops working just reestablish a porta_base register
;;; and start with that in W

OUTPUT:


;{
	
	; Push bits onto LED drivers
	
	banksel	PORTA

	movlw	0x04	
	btfsc	OUTPUT_H, 0 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 0	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	movlw	0x04	
	btfsc	OUTPUT_H, 1 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 1	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	movlw	0x04	
	btfsc	OUTPUT_H, 2 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 2	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	movlw	0x04	
	btfsc	OUTPUT_H, 3 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 3	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	movlw	0x04
	btfsc	OUTPUT_H, 4 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 4	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	movlw	0x04
	btfsc	OUTPUT_H, 5 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 5	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	movlw	0x04
	btfsc	OUTPUT_H, 6 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 6	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	movlw	0x04
	btfsc	OUTPUT_H, 7 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 7	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)
	
						 
	

	;iorlw	0x04	; Re-enable latching
	;movwf	PORTA

	RETURN			; Return from subroutine


;}










;;; ********************************************************************
;;; Subroutine Name:	INIT
;;; Description:	Initializes the messages in program memory
;;; Requires:	Nothing
;;; Returns:	Nothing
;;; Locations Affected:		We'll see	
;;; ********************************************************************

INIT:
;{
	;;; I'm shanghaing OUTPUT_L and OUTPUT_H for use for address registers here

	movlw 		0x0A
	movwf		OUTPUT_H
	movlw		0x00
	movwf		OUTPUT_L

	banksel		EECON1

	bsf			EECON1, WREN	; Enable writing

	banksel		EEADRH

	movf		OUTPUT_H, W		; Move the MSB of the address into EEADR
	movwf		EEADRH

	movf		OUTPUT_L, W		; Move the LSB of the address into EEADR
	movwf		EEADR


	;;;----------------------------------------------------------------	
	;;;	Paste the output from the string encoder below
	;;;----------------------------------------------------------------
;{
	; P
	movlw	0xff
	movwf	DATA1
	movlw	0xff
	movwf	DATA2
	movlw	0xff
	movwf	DATA3
	movlw	0xff
	movwf	DATA4
	call	INIT_WRITE

	; P
	movlw	0x81
	movwf	DATA1
	movlw	0x0
	movwf	DATA2
	movlw	0x81
	movwf	DATA3
	movlw	0x0
	movwf	DATA4
	call	INIT_WRITE

	; P
	movlw	0xc3
	movwf	DATA1
	movlw	0x0
	movwf	DATA2
	movlw	0x7e
	movwf	DATA3
	movlw	0x0
	movwf	DATA4
	call	INIT_WRITE

	; P
	movlw	0x3c
	movwf	DATA1
	movlw	0x0
	movwf	DATA2
	movlw	0x0
	movwf	DATA3
	movlw	0x0
	movwf	DATA4
	call	INIT_WRITE

	; A
	movlw	0x1f
	movwf	DATA1
	movlw	0xff
	movwf	DATA2
	movlw	0x7f
	movwf	DATA3
	movlw	0xff
	movwf	DATA4
	call	INIT_WRITE

	; A
	movlw	0xe0
	movwf	DATA1
	movlw	0x80
	movwf	DATA2
	movlw	0x80
	movwf	DATA3
	movlw	0x80
	movwf	DATA4
	call	INIT_WRITE

	; A
	movlw	0xe0
	movwf	DATA1
	movlw	0x80
	movwf	DATA2
	movlw	0x7f
	movwf	DATA3
	movlw	0xff
	movwf	DATA4
	call	INIT_WRITE

	; A
	movlw	0x1f
	movwf	DATA1
	movlw	0xff
	movwf	DATA2
	movlw	0x0
	movwf	DATA3
	movlw	0x0
	movwf	DATA4
	call	INIT_WRITE

	; T
	movlw	0xc0
	movwf	DATA1
	movlw	0x0
	movwf	DATA2
	movlw	0xc0
	movwf	DATA3
	movlw	0x0
	movwf	DATA4
	call	INIT_WRITE

	; T
	movlw	0xff
	movwf	DATA1
	movlw	0xff
	movwf	DATA2
	movlw	0xff
	movwf	DATA3
	movlw	0xff
	movwf	DATA4
	call	INIT_WRITE

	; T
	movlw	0xff
	movwf	DATA1
	movlw	0xff
	movwf	DATA2
	movlw	0xc0
	movwf	DATA3
	movlw	0x0
	movwf	DATA4
	call	INIT_WRITE

	; T
	movlw	0xc0
	movwf	DATA1
	movlw	0x0
	movwf	DATA2
	movlw	0x0
	movwf	DATA3
	movlw	0x0
	movwf	DATA4
	call	INIT_WRITE

	; R
	movlw	0xff
	movwf	DATA1
	movlw	0xff
	movwf	DATA2
	movlw	0xff
	movwf	DATA3
	movlw	0xff
	movwf	DATA4
	call	INIT_WRITE

	; R
	movlw	0x81
	movwf	DATA1
	movlw	0x0
	movwf	DATA2
	movlw	0x81
	movwf	DATA3
	movlw	0x80
	movwf	DATA4
	call	INIT_WRITE

	; R
	movlw	0xc3
	movwf	DATA1
	movlw	0xf0
	movwf	DATA2
	movlw	0x7e
	movwf	DATA3
	movlw	0x7f
	movwf	DATA4
	call	INIT_WRITE

	; R
	movlw	0x3c
	movwf	DATA1
	movlw	0xf
	movwf	DATA2
	movlw	0x0
	movwf	DATA3
	movlw	0x0
	movwf	DATA4
	call	INIT_WRITE

	; I
	movlw	0xc0
	movwf	DATA1
	movlw	0x3
	movwf	DATA2
	movlw	0xc0
	movwf	DATA3
	movlw	0x3
	movwf	DATA4
	call	INIT_WRITE

	; I
	movlw	0xff
	movwf	DATA1
	movlw	0xff
	movwf	DATA2
	movlw	0xff
	movwf	DATA3
	movlw	0xff
	movwf	DATA4
	call	INIT_WRITE

	; I
	movlw	0xff
	movwf	DATA1
	movlw	0xff
	movwf	DATA2
	movlw	0xc0
	movwf	DATA3
	movlw	0x3
	movwf	DATA4
	call	INIT_WRITE

	; I
	movlw	0xc0
	movwf	DATA1
	movlw	0x3
	movwf	DATA2
	movlw	0x0
	movwf	DATA3
	movlw	0x0
	movwf	DATA4
	call	INIT_WRITE

	; C
	movlw	0x3f
	movwf	DATA1
	movlw	0xfc
	movwf	DATA2
	movlw	0x7f
	movwf	DATA3
	movlw	0xfe
	movwf	DATA4
	call	INIT_WRITE

	; C
	movlw	0xc0
	movwf	DATA1
	movlw	0x3
	movwf	DATA2
	movlw	0x80
	movwf	DATA3
	movlw	0x1
	movwf	DATA4
	call	INIT_WRITE

	; C
	movlw	0x80
	movwf	DATA1
	movlw	0x1
	movwf	DATA2
	movlw	0xe0
	movwf	DATA3
	movlw	0x7
	movwf	DATA4
	call	INIT_WRITE

	; C
	movlw	0x60
	movwf	DATA1
	movlw	0x6
	movwf	DATA2
	movlw	0x0
	movwf	DATA3
	movlw	0x0
	movwf	DATA4
	call	INIT_WRITE

	; K
	movlw	0xff
	movwf	DATA1
	movlw	0xff
	movwf	DATA2
	movlw	0xff
	movwf	DATA3
	movlw	0xff
	movwf	DATA4
	call	INIT_WRITE

	; K
	movlw	0x1
	movwf	DATA1
	movlw	0x0
	movwf	DATA2
	movlw	0x3
	movwf	DATA3
	movlw	0xe0
	movwf	DATA4
	call	INIT_WRITE

	; K
	movlw	0x6
	movwf	DATA1
	movlw	0xfc
	movwf	DATA2
	movlw	0x1c
	movwf	DATA3
	movlw	0x1f
	movwf	DATA4
	call	INIT_WRITE

	; K
	movlw	0xf8
	movwf	DATA1
	movlw	0x3
	movwf	DATA2
	movlw	0xe0
	movwf	DATA3
	movlw	0x0
	movwf	DATA4
	call	INIT_WRITE
	
	movlw	0x7
	movwf	MSG_COUNT



;}




	;;;----------------------------------------------------------------	
	;;;	End of pasted code
	;;;----------------------------------------------------------------

	movf		MSG_COUNT, W	; Initialize the offset so that only
	sublw		0x1				; one character shows initially
	movlw		OFFSET			
	
	clrw		
	movwf		LEAD_SPACES		; No leading spaces to start
	
	movwf		ROTATIONS		; Start with 0 rotations
	
	RETURN
	;}
	
;;; --------------------------------------------------------------------
;;; Code-section Constant Data.
;;; --------------------------------------------------------------------


;;; ---------------------------------------------------------------------
;;; Vectors Section.
;;; ---------------------------------------------------------------------
;;; Establish the reset vector.  Note that the "0x0000" after the CODE
;;; directive ensures that the linker places the code at program memory
;;; location 0x0000.

Reset:	CODE	0x0000
	pagesel Main		; Selects page of program memory with.
	goto	Main		; the actual program, then goes to it.

	END			; End of file.

