        title ""
;;; *********************************************************************
;;; Filename:	main.asm
;;; Purpose:	Main loop for Rotary Display Control
;;; Revision:	0.1
;;; Date:	March 21, 2010
;;; Author:	Patrick Boyd
;;; Copyright:	
;;; *********************************************************************
;;; Revision History:
;;;	
;;; *********************************************************************


	


 	#include <p16f873a.inc>

SI1	equ	0x0
SI2	equ	0x1
LE	equ	0x2
OE	equ	0x3
CLOCK	equ	0x4

	
;;; --------------------------------------------------------------------
;;; Included files (usually library headers).  eg) #include "del_lib.h"
;;; --------------------------------------------------------------------
	

;;; --------------------------------------------------------------------
;;; Assembler Equates Section.  Define assembly-time constants here
;;;				using the EQU assembler directive.
;;;	eg)  DEBUGGING EQU  1
;;; --------------------------------------------------------------------

	
;;; ---------------------------------------------------------------------
;;; Variable Address Assignments.
;;; ---------------------------------------------------------------------
;	UDATA			; Start of uninitialized data section.
				; Reserve memory for variables using the
				; "RES" directive here.  Note that the
				; linker will assign addresses.
				; eg)  ADcounter  res  1
				;      (reserves 1 byte for ADcounter).



;	UDATA_OVR		; Uninitialized data overlay section.
				; Upon occasion, temporary variables
				; can be over-written by other temporary
				; variables.  These would appear here.
				; BUT!  If you ever intend on time-sliced
				; or pre-emptive multitasking		 
				; don't *ever* use this directive!

	;UDATA_SHR		; Uninitialized data shared section.
				; Variables that appear here appear
				; across all banks, meaning that no
				; bank selection will be required.
				; Variables of this type are *required*
				; when saving CPU context when an
				; interrupt occurs.  Not all devices
				; have shared memory.  In that case, you
				; need to reserve memory across individual
				; banks at the same offset by using
				; multiple UDATA or UDATA_SHR assignments
				; with an optional address specified.
W_Context:	res 1		; Context saving on interrupt:	W.

	;; For saving context.
STATUS_Context:	res 1		; Saving of STATUS on interrupt.
PCLATH_Context:	res 1		; Saving of PCLATH on interrupt.

	;;	Output to be displayed
OUTPUT_H:	res 1
OUTPUT_L:	res 1

	;; 	Registers for storing temp values for output
TEMP_H:		res 1
TEMP_L:		res 1

	;; Register for delay loop
COUNT:		res 1



;;; ---------------------------------------------------------------------
;;; Establish the OPTION register bit values. 
;;; ---------------------------------------------------------------------
;;; The '__CONFIG' directive is used to embed PIC configuration data 
;;; within an assembly file. The labels following the directive 
;;; are located in the .inc file.  See the device data sheet for
;;; additional information on the configuration word.

	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _WRT_ENABLE_ON & _LVP_OFF & _CPD_OFF

;;; _CP_OFF:	turn off code protection.  Don't change this unless you
;;;             want a device that can never be programmed again.  This
;;;             is a "bug" in some PIC devices.
;;; _WDT_OFF:	turn off the watchdog timer.
;;; _BODEN_ON:	turn on power brown-out reset.
;;; _PWRTE_ON:	turn on power-up timer.
;;; _XT_OSC:	specify that the device is using an XT oscillator.
;;; _WRT_ENABLE_ON:  enable writing to data EEPROM.
;;; _LVP_OFF:	disable low-voltage in-circuit programming.
;;; _CPD_OFF:	disable data EEPROM write protection.



;;; _CP_OFF:	turn off code protection.  Don't change this unless you
;;;             want a device that can never be programmed again.  This
;;;             is a "bug" in some PIC devices.
;;; _WDT_OFF:	turn off the watchdog timer.
;;; _BODEN_ON:	turn on power brown-out reset.
;;; _PWRTE_ON:	turn on power-up timer.
;;; _XT_OSC:	specify that the device is using an XT oscillator.
;;; _WRT_OFF:	disable write-protection of program FLASH.
;;; _LVP_OFF:	disable low-voltage in-circuit programming.
;;; _CPD_OFF:	disable data EEPROM write protection.
	
;;; --------------------------------------------------------------------
;;; Main Program.
;;; --------------------------------------------------------------------
ISR:	org	0x0004		; Location of interrupt vector.

	;; Save the CPU context.
	;; Note that W_Context was declared under the UDATA_SHR section.
	;; In the PIC16F876/7, it will appear across all banks of memory 
	;; and therefore no bank switching is required. For the
	;; PIC16F873/4, a linker script and use of the UDATA_SHR section
	;; reserves space for W_Context across all banks, even though
	;; there are actually two separate locations.  This code takes
	;; this possible separation into account.  See the linker script
	;; (16F873.lkr), available from the same location as this
	;; template for how this allocation occurs.
	
	movwf	W_Context
	swapf	STATUS,W
	banksel	STATUS_Context	
	movwf	STATUS_Context
	movf	PCLATH,W
	movwf	PCLATH_Context

	;; Why the swapf instruction, above?  That's a good question.
	;; The reason is that, unlike a move instruction, swapf doesn't
	;; affect any bits in STATUS, which is important!
	;; There are situations where you will need to save (and also
	;; restore more than just these two registers.

	;; ------------------------------------------------------------
	;; Your interrupt service routine code goes here.
	;; ------------------------------------------------------------
	;; You'll generally need to follow these steps:
	;; 1.  Determine what caused the interrupt by checking
	;;     appropriate interrupt flags.  You'll usually know which
	;;     ones to check, since you enable interrupts from certain
	;;     peripherals in the main program.
	;; 2.  Take appropriate action.
	;; 3.  Clear the interrupt source.  How to do this depends
	;;     on the peripheral that caused the interrupt, but sometimes
	;;     it is as simple as clearing a flag.
	;; ------------------------------------------------------------

	btfsc	PIR1, TMR1IF	; Check to see if it was the timer that caused the interrupt
	goto	TMR1_ISR

EXT_ISR:
	banksel PORTA
	btfsc	PORTA, OE
	goto	OE_OFF
	bsf		PORTA, OE
	bcf		INTCON, INTF
	goto	ISR_Restore
OE_OFF:
	bcf		PORTA, OE
	bcf		INTCON, INTF 
	goto	ISR_Restore

TMR1_ISR:
	banksel PORTA
	btfsc	PORTA, OE
	goto	OE_OFF
	bsf		PORTA, OE
	bcf		INTCON, INTF
	goto	ISR_Restore

;OE_OFF:
	;bcf		PORTA, OE
	;bcf		INTCON, INTF 
	;goto	ISR_Restore		;;; Uncomment this line if a new Interrupt handler is added



	;; Come back to this point when your interrupt processing is
	;; complete.
ISR_Restore:
	;; Restore the CPU context.
	banksel PCLATH_Context
	movf	PCLATH_Context,W
	movwf	PCLATH
	swapf	STATUS_Context,W
	movwf	STATUS		; Restores whichever bank W_Context
				; actually appears in.
	swapf	W_Context,F	; See the note above as to why swapf
	swapf	W_Context,W	; is used.

	;; Return from the interrupt, which pulls an address off the
	;; (invisible) stack.
	retfie

	;; ------------------------------------------------------------	
	;; Your main code starts here.  If it just follows the ISR, that
	;; is usually OK.
	;; ------------------------------------------------------------	
	

Main:	org 0x200
	;; Program initialization.
	banksel TRISB 	; Move to the bank with TRISB
	movlw	0x01 	; Configure all bits of RB to be outputs
	movwf 	TRISB 	; except for RB0 which is the interrupt line

	;movlw	0x1F	; Configure the first 5 pins of RA to be ouput
	;movwf 	TRISA

	clrf	TRISA	; Configure all RA pins to be output
	
	;; Clear the interrupt flag for the individual peripheral(s),
	;; just in case there is already an outstanding interrupt.
	;; This is usually in a PIR register.
	bcf INTCON, INTF ; clear the RB0 interupt bit just in case
	
	;; Enable interrupts from individual peripheral(s).
	;; This is usually in a PIE register.
	bsf INTCON, INTE 	;   enable interrupts on RB0
	bsf INTCON, PEIE	;	and for peripherals
	banksel OPTION_REG
	bcf OPTION_REG, INTEDG ; Cause interrupts to be triggered on a falling edge
	clrf 	PIE1	; clear the peripheral interrupt fields
	bsf		PIE1, TMR1IE	; enable the timer1 overflow interrupt

	;; Turn on interrupts (in general -- this is the overall
	;; interrupt "power switch").  This is usually done at
	;; this location: after the rest of initialization but
	;; just before the program goes into its overall loop.
	bsf	INTCON, GIE	; INTCON appears in all banks.

	;; Set up timer1 to track the rotational speed of the arm

	banksel T1CON
	clrf	T1CON
	clrf	TMR1H
	clrf	TMR1L
	clrf	PIR1

	banksel	PORTA 	; Select the bank with PORTA, since we'll be using it

	;bsf		T1CON, TMR1ON	; Start the Timer
	
;;;	-------------------------------------------------------------------------------------
;;;	For Reference Here are the list of which pins on the PIC
;;;	are associated with which LED driver lines
;;;	RB0 - Hall Sensor
;;;	RA0 - Serial In 1
;;;	RA1 - Serial In 2
;;; RA2 - Latch Enable
;;;	RA3 - Output Enable
;;;	RA4 - Clock
;;; -------------------------------------------------------------------------------------


	bcf		PORTA, OE		;; Set output enable low
	bsf		PORTA, LE		;; Set Latch enable high
	bsf		PORTA, SI1
	bsf		PORTA, SI2
	bcf		PORTA, SI1
	bcf		PORTA, SI2
	bsf		PORTA, LE
	movlw		0x03
	iorwf		PORTA, 1

Debug:
	movlw		0x07
	movwf		PORTA
	
	bsf		PORTA, CLOCK
	bcf		PORTA, CLOCK


DebugLoop:
	NOP
	NOP
	goto DebugLoop
	

MainLoop:

	movlw	0x01
	movwf	OUTPUT_H
	movlw	0x00
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x02
	movwf	OUTPUT_H
	movlw	0x00
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x04
	movwf	OUTPUT_H
	movlw	0x00
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x08
	movwf	OUTPUT_H
	movlw	0x00
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x10
	movwf	OUTPUT_H
	movlw	0x00
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x20
	movwf	OUTPUT_H
	movlw	0x00
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x40
	movwf	OUTPUT_H
	movlw	0x00
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x80
	movwf	OUTPUT_H
	movlw	0x00
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x00
	movwf	OUTPUT_H
	movlw	0x01
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x00
	movwf	OUTPUT_H
	movlw	0x02
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x00
	movwf	OUTPUT_H
	movlw	0x04
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x00
	movwf	OUTPUT_H
	movlw	0x08
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x00
	movwf	OUTPUT_H
	movlw	0x10
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x00
	movwf	OUTPUT_H
	movlw	0x20
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x00
	movwf	OUTPUT_H
	movlw	0x40
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	movlw	0x00
	movwf	OUTPUT_H
	movlw	0x80
	movwf	OUTPUT_L
	call	OUTPUT
	call 	DELAY

	goto MainLoop ; Return to the start of the loop

	;; Don't let execution fall down to here, otherwise the
	;; subroutines (or whatever follows) will be executed without
	;; parameters, and chaos reigns when a "return" instruction
	;; executes with no return address on the stack!
	
;;; --------------------------------------------------------------------
;;; Application-specific Subroutines.
;;; --------------------------------------------------------------------
;;; ********************************************************************
;;; Subroutine Name:	
;;; Description:	
;;; Requires:		
;;; Returns:		
;;; Locations Affected:	
;;; ********************************************************************


;;; ********************************************************************
;;; Subroutine Name: DELAY
;;; Description: Runs through a busy loop to delay a bit	
;;; Requires: Nothing
;;; Returns: Nothing
;;; Locations Affected:	W?
;;; ********************************************************************

DELAY:

	movlw	0xff
	movwf	COUNT

DELAY_LOOP:
	decfsz	COUNT, 1
	goto	DELAY_LOOP

	RETURN

;;; ********************************************************************
;;; Subroutine Name: OUTPUT	
;;; Description: Outputs the values stored in OUTPUT_H and OUTPUT_L to the LEDs
;;; Requires: -The values of OUPUT_H and OUTPUT_L to be set
;;;			  -PORTB bank to be selected
;;; Returns: Nothing
;;; Locations Affected:	W, TEMP_H, TEMP_L
;;; ********************************************************************

;;; Right now this is taking 10 Cycles per 2 LEDs, which is 80 cycles for the while thing.
;;; Check later to see if that is too much...

OUTPUT:
	;bcf	PORTA, LE ; Clear Latch Enable bit (hopefully, I'm not sure this syntax works)
	
	; Push bits onto LED drivers

	bcf		PORTA, SI1	; Clear the Serial In 1 bit initially
	bcf		PORTA, SI2	; Clear the Serial In 2 bit initially
	btfsc	OUTPUT_H, 0 ; Test the first bit in OUTPUT_H
	bsf		PORTA, SI1	; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 0	; Test the first bit in OUTPUT_L
	bsf		PORTA, SI2	; Set it high if the output bit is 1
	bcf		PORTA, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bsf		PORTA, CLOCK	;	but I might need to add a delay in here)

	bcf		PORTA, SI1	; Clear the Serial In 1 bit initially
	bcf		PORTA, SI2	; Clear the Serial In 2 bit initially
	btfsc	OUTPUT_H, 1 ; Test the first bit in OUTPUT_H
	bsf		PORTA, SI1	; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 1	; Test the first bit in OUTPUT_L
	bsf		PORTA, SI2	; Set it high if the output bit is 1
	bcf		PORTA, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bsf		PORTA, CLOCK	;	but I might need to add a delay in here)

	bcf		PORTA, SI1	; Clear the Serial In 1 bit initially
	bcf		PORTA, SI2	; Clear the Serial In 2 bit initially
	btfsc	OUTPUT_H, 2 ; Test the first bit in OUTPUT_H
	bsf		PORTA, SI1	; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 2	; Test the first bit in OUTPUT_L
	bsf		PORTA, SI2	; Set it high if the output bit is 1
	bcf		PORTA, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bsf		PORTA, CLOCK	;	but I might need to add a delay in here)

	bcf		PORTA, SI1	; Clear the Serial In 1 bit initially
	bcf		PORTA, SI2	; Clear the Serial In 2 bit initially
	btfsc	OUTPUT_H, 3 ; Test the first bit in OUTPUT_H
	bsf		PORTA, SI1	; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 3	; Test the first bit in OUTPUT_L
	bsf		PORTA, SI2	; Set it high if the output bit is 1
	bcf		PORTA, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bsf		PORTA, CLOCK	;	but I might need to add a delay in here)

	bcf		PORTA, SI1	; Clear the Serial In 1 bit initially
	bcf		PORTA, SI2	; Clear the Serial In 2 bit initially
	btfsc	OUTPUT_H, 4 ; Test the first bit in OUTPUT_H
	bsf		PORTA, SI1	; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 4	; Test the first bit in OUTPUT_L
	bsf		PORTA, SI2	; Set it high if the output bit is 1
	bcf		PORTA, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bsf		PORTA, CLOCK	;	but I might need to add a delay in here)

	bcf		PORTA, SI1	; Clear the Serial In 1 bit initially
	bcf		PORTA, SI2	; Clear the Serial In 2 bit initially
	btfsc	OUTPUT_H, 5 ; Test the first bit in OUTPUT_H
	bsf		PORTA, SI1	; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 5	; Test the first bit in OUTPUT_L
	bsf		PORTA, SI2	; Set it high if the output bit is 1
	bcf		PORTA, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bsf		PORTA, CLOCK	;	but I might need to add a delay in here)

	bcf		PORTA, SI1	; Clear the Serial In 1 bit initially
	bcf		PORTA, SI2	; Clear the Serial In 2 bit initially
	btfsc	OUTPUT_H, 6 ; Test the first bit in OUTPUT_H
	bsf		PORTA, SI1	; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 6	; Test the first bit in OUTPUT_L
	bsf		PORTA, SI2	; Set it high if the output bit is 1
	bcf		PORTA, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bsf		PORTA, CLOCK	;	but I might need to add a delay in here)

	bcf		PORTA, SI1	; Clear the Serial In 1 bit initially
	bcf		PORTA, SI2	; Clear the Serial In 2 bit initially
	btfsc	OUTPUT_H, 7 ; Test the first bit in OUTPUT_H
	bsf		PORTA, SI1	; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 7	; Test the first bit in OUTPUT_L
	bsf		PORTA, SI2	; Set it high if the output bit is 1
	bcf		PORTA, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bsf		PORTA, CLOCK	;	but I might need to add a delay in here)
	
						 
	

	;bsf 	PORTA, LE	; Enable Latching again to display new pattern

	RETURN			; Return from subroutine

	
;;; --------------------------------------------------------------------
;;; Code-section Constant Data.
;;; --------------------------------------------------------------------


;;; ---------------------------------------------------------------------
;;; Vectors Section.
;;; ---------------------------------------------------------------------
;;; Establish the reset vector.  Note that the "0x0000" after the CODE
;;; directive ensures that the linker places the code at program memory
;;; location 0x0000.

Reset:	org	0x0000
	;pagesel Main		; Selects page of program memory with.
	goto	Main		; the actual program, then goes to it.

	END			; End of file.

